@@ -4,7 +4,7 @@ use warnings;
 use vars qw( $VERSION );
 use Parse::HTTP::UserAgent::Constants qw(:all);
 
-$VERSION = '0.34';
+$VERSION = '0.37';
 
 sub _extract_dotnet {
     my($self, @args) = @_;
@@ -18,7 +18,8 @@ sub _extract_dotnet {
         }
         if ( $e =~ RE_WINDOWS_OS ) {
             if ( $1 && $1 ne '64' ) {
-                $self->[UA_OS] = $e;
+                # Maxthon stupidity: multiple OS definitions
+                $self->[UA_OS] ||= $e;
                 next;
             }
         }
@@ -64,13 +65,44 @@ sub _fix_generic {
 
 sub _parse_maxthon {
     my($self, $moz, $thing, $extra, @others) = @_;
-    my @omap = grep { $_ } map { split RE_SC_WS_MULTI, $_ } @others;
+    my $is_30 =    $extra
+                && $extra->[0]
+                && index( $extra->[0], 'AppleWebKit' ) != NO_IMATCH;
     my($maxthon, $msie, @buf);
+
+    if ( $is_30 ) {
+        # yay, new nonsense with the new version
+        my @new;
+        for my $i (0..$#others) {
+            if ( index( $others[$i], 'Maxthon') != NO_IMATCH ) {
+                @new        = split m{\s+}xms, $others[$i];
+                $maxthon    = shift @new;
+                $extra    ||= [];
+                unshift @{ $extra }, shift @new;
+                $others[$i] = '';
+                last;
+            }
+        }
+        @others = grep { $_ } @others, @new;
+        $self->_parse_safari( $moz, $thing, $extra, @others );
+        $self->[UA_NAME] = 'Maxthon';
+    }
+    else {
+    my @omap = grep { $_ } map { split RE_SC_WS_MULTI, $_ } @others;
+
     foreach my $e ( @omap, @{$thing} ) { # $extra -> junk
-        if ( index(uc $e, 'MAXTHON') != NO_IMATCH ) { $maxthon = $e; next; }
-        if ( index(uc $e, 'MSIE'   ) != NO_IMATCH ) { $msie    = $e; next; }
+        if ( index(uc $e, 'MAXTHON') != NO_IMATCH ) {
+            $maxthon = $e;
+            next;
+        }
+        if ( index(uc $e, 'MSIE' ) != NO_IMATCH ) {
+            # Maxthon stupidity: multiple MSIE strings
+            $msie ||= $e;
+            next;
+        }
         push @buf, $e;
     }
+    }
 
     if ( ! $maxthon ) {
         warn ERROR_MAXTHON_VERSION . "\n";
@@ -78,17 +110,24 @@ sub _parse_maxthon {
         return;
     }
 
-    if ( ! $msie ) {
-        warn ERROR_MAXTHON_MSIE . "\n";
-        $self->[UA_UNKNOWN] = 1;
-        return;
+    if ( $is_30 ) {
+        if ( $self->[UA_LANG] ) {
+            push @{ $self->[UA_EXTRAS] }, $self->[UA_LANG];
+            $self->[UA_LANG] = undef;
+        }
+    }
+    else {
+        if ( ! $msie ) {
+            warn ERROR_MAXTHON_MSIE . "\n";
+            $self->[UA_UNKNOWN] = 1;
+            return;
+        }
+        $self->_parse_msie(
+            $moz, [ undef, @buf ], undef, split RE_WHITESPACE, $msie
+        );
     }
 
-    $self->_parse_msie(
-        $moz, [ undef, @buf ], undef, split RE_WHITESPACE, $msie
-    );
-
-    my(undef, $mv) = split RE_WHITESPACE, $maxthon;
+    my(undef, $mv) = split $is_30 ? RE_SLASH : RE_WHITESPACE, $maxthon;
     my $v = $mv      ? $mv
           : $maxthon ? '1.0'
           :            do { warn ERROR_MAXTHON_VERSION . "\n"; 0 }
@@ -96,6 +135,7 @@ sub _parse_maxthon {
 
     $self->[UA_ORIGINAL_VERSION] = $v;
     $self->[UA_ORIGINAL_NAME]    = 'Maxthon';
+    $self->[UA_PARSER]           = 'maxthon';
     return 1;
 }
 
@@ -124,18 +164,58 @@ sub _parse_msie {
         }
         push @buf, $e;
     }
-    $self->[UA_EXTRAS] = [ @buf ];
+
+    $self->[UA_EXTRAS] = [
+        map  { $self->trim( $_ ) }
+        grep { $_ !~ m{ \s+ compatible \z }xms }
+        @buf
+    ];
+
     $self->[UA_PARSER] = 'msie';
+
     return 1;
 }
 
 sub _parse_firefox {
     my($self, @args) = @_;
     $self->_parse_mozilla_family( @args );
+    my $e = $self->[UA_EXTRAS];
+
+    if ( ref $e eq 'ARRAY'
+        && @{ $e } > 0
+        && index( lc $e->[-1], 'fennec' ) != NO_IMATCH
+    ) {
+        $self->_fix_fennec( $e );
+    }
+
     $self->[UA_NAME] = 'Firefox';
+
     return 1;
 }
 
+sub _fix_fennec {
+    my($self, $e) = @_;
+    my($name, $version) = split RE_SLASH, pop @{ $e };
+    $self->[UA_ORIGINAL_NAME]    = $name;
+    $self->[UA_ORIGINAL_VERSION] = $version;
+    $self->[UA_MOBILE]           = 1;
+    return if ! $self->[UA_LANG];
+
+    if ( lc $self->[UA_LANG] eq 'tablet' ) {
+        $self->[UA_TABLET] = 1;
+        $self->[UA_LANG]   = undef;
+    }
+    elsif ( index( $self->[UA_LANG], q{ } ) != NO_IMATCH ) {
+        push @{ $self->[UA_EXTRAS] }, $self->[UA_LANG];
+        $self->[UA_LANG] = undef;
+    }
+    else {
+        # Do nothing
+    }
+
+    return;
+}
+
 sub _parse_safari {
     my($self, $moz, $thing, $extra, @others) = @_;
     my $ipad            = $thing && lc( $thing->[0] || q{} ) eq 'ipad';
@@ -158,7 +238,7 @@ sub _parse_safari {
                             : $ipad ? 'iPad'
                             :         'Safari';
     $self->[UA_VERSION_RAW] = $vx;
-    $self->[UA_TOOLKIT]     = $extra ? [ split RE_SLASH, $extra->[0] ] : [];
+    $self->[UA_TOOLKIT]     = $extra ? [ split RE_SLASH, shift @{ $extra } ] : [];
     if ( $thing->[-1] && length($thing->[LAST_ELEMENT]) <= 5 ) {
         # todo: $self->_is_lang_field($junk)
         # in here or in _post_parse()
@@ -202,18 +282,29 @@ sub _parse_safari {
     }
 
     push @{$self->[UA_EXTRAS]}, @junk if @junk;
+    push @{$self->[UA_EXTRAS]}, @{$extra} if $extra;
 
     return 1;
 }
 
 sub _parse_chrome {
     my($self, $moz, $thing, $extra, @others) = @_;
-    my $chx                  = pop @others;
-    my($chrome, $safari)     = split RE_WHITESPACE, $chx;
-    push @others, $safari;
+    my $chx = pop @others;
+    my($chrome, $safari, @rest) = split RE_WHITESPACE, $chx;
+    my $opera;
+    if ( $rest[0] && index( $rest[0], 'OPR/', 0) != NO_IMATCH ) {
+        $opera = shift @rest;
+        if ( ref $extra eq 'ARRAY' ) {
+            unshift @{ $extra }, $chrome;
+        }
+        push @others, @rest, $safari;
+    }
+    else {
+        push @others, $safari;
+    }
     $self->_parse_safari($moz, $thing, $extra, @others);
-    my($name, $version)      = split RE_SLASH, $chrome;
-    $self->[UA_NAME]         = $name;
+    my($name, $version)      = split RE_SLASH, $opera || $chrome;
+    $self->[UA_NAME]         = $opera ? 'Opera' : $name;
     $self->[UA_VERSION_RAW]  = $version;
     return 1;
 }
@@ -221,6 +312,13 @@ sub _parse_chrome {
 sub _parse_android {
     my($self, $moz, $thing, $extra, @others) = @_;
     (undef, @{$self}[UA_STRENGTH, UA_OS, UA_LANG, UA_DEVICE]) = @{ $thing };
+    if ( ! $extra
+        && $others[0]
+        && index( $others[0], 'AppleWebKit' ) != NO_IMATCH
+    ) {
+        $extra = [ shift @others ];
+        $self->[UA_PARSER] = 'android:paren_fixer';
+    }
     $self->[UA_TOOLKIT] = [ split RE_SLASH, $extra->[0] ] if $extra;
     my(@extras, $is_phone);
 
@@ -266,7 +364,7 @@ sub _parse_android {
     $self->[UA_NAME]   = 'Android';
     $self->[UA_MOBILE] = 1;
     $self->[UA_TABLET] = $is_phone ? undef : 1;
-    $self->[UA_EXTRAS] = [ @extras ];
+    $self->[UA_EXTRAS] = [ grep { $_ } @extras ];
 
     return 1;
 }
@@ -333,14 +431,20 @@ sub _parse_opera_post {
 }
 
 sub _parse_mozilla_family {
-    my($self, $moz, $thing, $extra, @extras) = @_;
+    my($self, $moz, $thing, $extra, @others) = @_;
     # firefox variation or just mozilla itself
     my($name, $version)      = split RE_SLASH, defined $extra->[1] ? $extra->[1]
                              :                                       $moz
                              ;
+    if ( $version ) {
+        $extra->[1] = '';
+    }
     $self->[UA_NAME]         = $name;
-    $self->[UA_TOOLKIT]      = $extra ? [ split RE_SLASH, $extra->[0] ] : [];
     $self->[UA_VERSION_RAW]  = $version;
+    $self->[UA_TOOLKIT]      = $extra->[0]
+                             ? [ split RE_SLASH, shift @{ $extra } ]
+                             : []
+                             ;
 
     if ( @{$thing} && index($thing->[LAST_ELEMENT], 'rv:') != NO_IMATCH ) {
         $self->[UA_MOZILLA]  = pop @{ $thing };
@@ -357,7 +461,12 @@ sub _parse_mozilla_family {
         }
     }
 
-    $self->[UA_EXTRAS] = [ @{ $thing }, @extras ];
+    $self->[UA_EXTRAS] = [
+        grep { $_ }
+        @{ $thing },
+        @others,
+        $extra ? @{ $extra } : (),
+    ];
     return 1;
 }
 
@@ -568,6 +677,7 @@ sub _parse_emacs {
     $self->[UA_NAME]        = $name;
     $self->[UA_VERSION_RAW] = $version || 0;
     $self->[UA_OS]          = shift @{ $thing };
+    $self->[UA_OS]          = $self->trim( $self->[UA_OS] ) if $self->[UA_OS];
     my @rest = (  @{ $thing }, @moz );
     push @rest, @{ $extra } if $extra && ref $extra eq 'ARRAY';
     push @rest, ( map { split RE_SC_WS, $_ } @others ) if @others;
@@ -639,8 +749,8 @@ Parse::HTTP::UserAgent::Base::Parsers - Base class
 
 =head1 DESCRIPTION
 
-This document describes version C<0.34> of C<Parse::HTTP::UserAgent::Base::Parsers>
-released on C<8 April 2012>.
+This document describes version C<0.37> of C<Parse::HTTP::UserAgent::Base::Parsers>
+released on C<16 September 2013>.
 
 Internal module.
 
@@ -654,12 +764,11 @@ Burak Gursoy <burak@cpan.org>.
 
 =head1 COPYRIGHT
 
-Copyright 2009 - 2012 Burak Gursoy. All rights reserved.
+Copyright 2009 - 2013 Burak Gursoy. All rights reserved.
 
 =head1 LICENSE
 
-This library is free software; you can redistribute it and/or modify 
-it under the same terms as Perl itself, either Perl version 5.12.3 or, 
+This library is free software; you can redistribute it and/or modify
+it under the same terms as Perl itself, either Perl version 5.16.2 or,
 at your option, any later version of Perl 5 you may have available.
-
 =cut
